    MODULE data_in_struct_mod
!!!-----------------------------------------------------------------------
!!!
!!! Converts structures read from data to those used in the calculation
!!!
!!! ITEMS THAT NEED TO BE SET BY THE PROGRAM READING THE DATA
!!!
!!! progenitors_in -- sizes of the two progenitor alleles
!!!
!!! n_allele_in -- number of differently labelled alleles in data
!!!
!!! n_dna_in -- number of runs in data
!!!
!!! dna_in -- amount of dna for each run in experimenter units
!!!
!!! n_well_in -- number of wells (one value for each run)
!!!
!!! seen_in -- number of times each allele seen in each run
!!!
!!! size_in -- sizes of alleles
!!!
!!!-----------------------------------------------------------------------
! .. Default Accessibility ..
      PUBLIC
! ..
! .. Private Statements ..
      PRIVATE :: dpkind
! ..
! .. Local Arrays ..
      INTEGER :: progenitors_in(2)
! ..
! .. Parameters ..
      INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
      INTEGER, PARAMETER :: max_allele_in = 50, max_dna_in = 50
! ..
! .. Local Scalars ..
      INTEGER :: n_allele_in, n_allele_out, n_dna_in, tot_seen_in
! ..
! .. Intrinsic Functions ..
      INTRINSIC KIND
! ..
! .. Dependents ..
      REAL (dpkind) :: dna_in(max_dna_in)
      INTEGER :: n_well_in(max_dna_in), seen_in(max_dna_in,max_allele_in) &
        , size_in(max_allele_in)
! ..
    CONTAINS

!*********************************************************************

      SUBROUTINE values_to_structures
! .. Use Statements ..
        USE structures_mod
        USE print_array_mod
! ..
! .. Local Scalars ..
        INTEGER :: ix, i_allele, i_dna
! ..
! .. Intrinsic Functions ..
        INTRINSIC REAL, SUM
! ..
! .. Parameters ..
        REAL (dpkind), PARAMETER :: two = 2.0_dpkind
! ..
! .. Local Arrays ..
        INTEGER :: ix_allele_out(max_allele_in)
! ..
!!! Get rid of allele info if the allele was never seen
        n_allele_out = 0

        DO i_allele = 1, n_allele_in

          tot_seen_in = SUM(seen_in(:,i_allele))

          IF (tot_seen_in>0) THEN

            n_allele_out = n_allele_out + 1

            ix_allele_out(n_allele_out) = i_allele

          END IF

        END DO

!!! Allocate everything

        n_dna = n_dna_in

        n_allele = n_allele_out

        CALL initialize

!!! Put the info where it belongs

        n_well = n_well_in(1:n_dna_in)

!!! Change dna from g.e. to allele equivalents

        dna = two*dna_in(1:n_dna_in)

!!! Progenitor size => progenitor index

        progenitor = -1

ALLELE_LOOP: DO i_allele = 1, n_allele_out

          ix = ix_allele_out(i_allele)

          allele_size(i_allele) = size_in(ix)

          IF (progenitors_in(1)==allele_size(i_allele)) progenitor(1) &
            = i_allele

          IF (progenitors_in(2)==allele_size(i_allele)) progenitor(2) &
            = i_allele

          seen(:,i_allele,data) = REAL(seen_in(1:n_dna_in,ix), &
            kind=dpkind)

        END DO ALLELE_LOOP

        DO i_dna = 1, n_dna_in

          unseen(i_dna,:,data) = n_well_in(i_dna) - seen(i_dna,:,data)

        END DO

      END SUBROUTINE values_to_structures

!*********************************************************************

    END MODULE data_in_struct_mod
