    MODULE file_mod
! .. Implicit None Statement ..
      IMPLICIT NONE
! ..
! .. Default Accessibility ..
      PRIVATE
! ..
! .. Public Statements ..
      PUBLIC :: file_exists, parse_file_name
! ..
    CONTAINS

!*********************************************************************

      SUBROUTINE file_exists(file_name,exists,readable,writeable)
!!! Returns whether file exists and is readable or writeable
!!! For UNKNOWN will return unreadable or unwriteable
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Parameters ..
        CHARACTER (*), PARAMETER :: no = 'NO', yes = 'YES'
! ..
! .. Scalar Arguments ..
        LOGICAL, INTENT (OUT) :: exists
        LOGICAL, OPTIONAL, INTENT (OUT) :: readable, writeable
        CHARACTER (*), INTENT (IN) :: file_name
! ..
! .. Local Scalars ..
        CHARACTER (3) :: read_ynu_string, write_ynu_string
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
! .. Executable Statements ..

        INQUIRE (file=file_name,exist=exists,read=read_ynu_string, &
          write=write_ynu_string)

        IF (PRESENT(readable)) THEN
          readable = .FALSE.
          IF (read_ynu_string==yes) readable = .TRUE.
        END IF

        IF (PRESENT(writeable)) THEN
          writeable = .FALSE.
          IF (write_ynu_string/=no) writeable = .TRUE.
        END IF

      END SUBROUTINE file_exists

!*********************************************************************

      SUBROUTINE parse_file_name(string,path,name,ext)
!!! Breaks up file name in string into path, name, ext
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Parameters ..
        CHARACTER (*), PARAMETER :: ext_del = '.', path_del = '\:/'
! ..
! .. Scalar Arguments ..
        CHARACTER (*), INTENT (OUT) :: ext, name, path
        CHARACTER (*), INTENT (IN) :: string
! ..
! .. Local Scalars ..
        INTEGER :: ix
        CHARACTER (256) :: name_part
! ..
! .. Intrinsic Functions ..
        INTRINSIC LEN_TRIM, SCAN
! ..
! .. Executable Statements ..

        ix = SCAN(string,path_del,back=.TRUE.)

        IF (ix==0) THEN ! No path
          path = ''
          name_part = string
        ELSE IF (ix<LEN_TRIM(string)) THEN ! Both path and name_part
          path = string(:ix)
          name_part = string(ix+1:)
        ELSE !No name part
          path = string
          name = ''
          ext = ''
          RETURN
        END IF

        ix = SCAN(name_part,ext_del,back=.TRUE.)

        IF (ix==0 .OR. ix==LEN_TRIM(name_part)) THEN ! No extension
          name = name_part
          ext = ''
        ELSE IF (ix==1) THEN !No name
          name = ''
          ext = name_part
        ELSE IF (ix<LEN_TRIM(name_part)) THEN ! Both path and name_part
          name = name_part(:ix-1)
          ext = name_part(ix+1:)
        END IF

      END SUBROUTINE parse_file_name

!*********************************************************************

    END MODULE file_mod
