    MODULE fit_freq_mod
! .. Use Statements ..
      USE print_it
      USE structures_mod
      USE sppcr_aux_mod
! ..
! .. Default Accessibility ..
      PUBLIC
! ..
! .. Private Statements ..
      PRIVATE :: dpkind
! ..
! .. Parameters ..
      INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
! ..
    CONTAINS

!*********************************************************************

      SUBROUTINE est_var_calibrate(est_mu,var_mu,est_calibrate, &
          var_calibrate,sd_calibrate)
!!! NOTE: var_calibrate and sd_calibrate are calculated iff
!!! var_mu is present
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (OUT) :: est_calibrate
        REAL (dpkind), OPTIONAL, INTENT (OUT) :: sd_calibrate, &
          var_calibrate
! ..
! .. Array Arguments ..
        REAL (dpkind), INTENT (IN) :: est_mu(:)
        REAL (dpkind), OPTIONAL, INTENT (IN) :: var_mu(:)
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT, SQRT, SUM
! ..
        est_calibrate = SUM(est_mu)

        IF (PRESENT(var_calibrate)) THEN
          var_calibrate = SUM(var_mu)
          sd_calibrate = SQRT(var_calibrate)
        END IF

        RETURN

      END SUBROUTINE est_var_calibrate

!*********************************************************************

      SUBROUTINE est_var_freq(est_mu,var_mu,est_freq,var_freq,sd_freq)
!!! Computes estimate and variance of allele frequency from same
!!! for the mu's
!!! NOTE: var_freq and sd_freq are calculated only if var_mu is present
! .. Array Arguments ..
        REAL (dpkind), INTENT (OUT) :: est_freq(:)
        REAL (dpkind), INTENT (IN) :: est_mu(:)
        REAL (dpkind), OPTIONAL, INTENT (OUT) :: sd_freq(:), var_freq(:)
        REAL (dpkind), OPTIONAL, INTENT (IN) :: var_mu(:)
! ..
! .. Local Scalars ..
        REAL (dpkind) :: tot_mu_est, tot_mu_var, varx, vary, x, y
        INTEGER :: i_allele
        LOGICAL :: calc_var
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT, SQRT, SUM
! ..
        calc_var = PRESENT(var_mu)

!!! x is the estimate of the current allele
!!! y is all other alleles

        tot_mu_est = SUM(est_mu)

        IF (calc_var) tot_mu_var = SUM(var_mu)

        DO i_allele = 1, n_allele

          x = est_mu(i_allele)

          est_freq(i_allele) = x/tot_mu_est

          IF (calc_var) THEN

            y = tot_mu_est - x

            varx = var_mu(i_allele)

            vary = tot_mu_var - varx

            var_freq(i_allele) = (y**2*varx+x**2*vary)/tot_mu_est**4

            sd_freq(i_allele) = SQRT(var_freq(i_allele))

          END IF

        END DO

        RETURN

      END SUBROUTINE est_var_freq

!*********************************************************************

      SUBROUTINE est_var_mut(est_mu,var_mu,est_mut_freq,var_mut_freq, &
          sd_mut_freq)
!!! NOTE: VAR AND SD CALCULATED ONLY IF VAR_MU IS PRESENT
! .. Array Arguments ..
        REAL (dpkind), INTENT (IN) :: est_mu(:)
        REAL (dpkind), OPTIONAL, INTENT (IN) :: var_mu(:)
! ..
! .. Local Scalars ..
        REAL (dpkind) :: tot_mu_est, tot_mu_var, varx, vary, x, y
        LOGICAL :: calc_var, heterozygous
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT, SQRT, SUM
! ..
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (OUT) :: est_mut_freq
        REAL (dpkind), OPTIONAL, INTENT (OUT) :: sd_mut_freq, &
          var_mut_freq
! ..
        calc_var = PRESENT(var_mu)

        heterozygous = progenitor(1) /= progenitor(2)

!!! x is the estimate the total of all alleles except for progenitor alleles
!!! y is progenitor alleles

        tot_mu_est = SUM(est_mu)

        IF (calc_var) tot_mu_var = SUM(var_mu)

        IF (heterozygous) THEN
          x = tot_mu_est - est_mu(progenitor(1)) - est_mu(progenitor(2))
        ELSE
          x = tot_mu_est - est_mu(progenitor(1))
        END IF

        est_mut_freq = x/tot_mu_est

DO_VAR: IF (calc_var) THEN

          IF ( .NOT. heterozygous) THEN

            y = tot_mu_est - x

            varx = tot_mu_var - var_mu(progenitor(1))

            vary = var_mu(progenitor(1))

          ELSE ! Heterozygous

            y = tot_mu_est - x

            varx = tot_mu_var - var_mu(progenitor(1)) - &
              var_mu(progenitor(2))

            vary = var_mu(progenitor(1)) + var_mu(progenitor(2))

          END IF

          var_mut_freq = (y**2*varx+x**2*vary)/tot_mu_est**4

          sd_mut_freq = SQRT(var_mut_freq)

        END IF DO_VAR

        RETURN

      END SUBROUTINE est_var_mut

!*********************************************************************

    END MODULE fit_freq_mod
