    MODULE fit_mu_mod
! .. Use Statements ..
      USE structures_mod
! ..
! .. Default Accessibility ..
      PRIVATE
! ..
! .. Public Statements ..
      PUBLIC :: fit_all_mu
! ..
! .. Parameters ..
      INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
      INTEGER, PARAMETER :: max_allele = 100, max_dna = 100
      REAL (dpkind), PARAMETER :: huge_mu = 1.0E10_dpkind
      REAL (dpkind), PARAMETER :: hundredth = 0.01_dpkind
      REAL (dpkind), PARAMETER :: one = 1.0_dpkind
      REAL (dpkind), PARAMETER :: tiny = 1.0E-100_dpkind
      REAL (dpkind), PARAMETER :: tiny_mu = 1.0E-10_dpkind
      REAL (dpkind), PARAMETER :: zero = 0.0_dpkind
! ..
! .. Local Arrays ..
      REAL (dpkind) :: mean(max_dna), pseen(max_dna), punseen(max_dna), &
        punseen_d_pseen(max_dna), tmp(max_dna)
! ..
    CONTAINS

!*********************************************************************

      ELEMENTAL FUNCTION expm1(x)
!----------------------------------------------------------------------
!      DOUBLE PRECISION FUNCTION expm1(x)
!            Evaluation of the function EXP(X) - 1
!                              Method
!     Renaming of function rexp from code of:
!     DiDinato, A. R. and Morris,  A.   H.  Algorithm 708: Significant
!     Digit Computation of the Incomplete  Beta  Function Ratios.  ACM
!     Trans. Math.  Softw. 18 (1993), 360-373.
!----------------------------------------------------------------------
! .. Parameters ..
        REAL (dpkind), PARAMETER :: half = 0.5_dpkind
        REAL (dpkind), PARAMETER :: one = 1.0_dpkind
        REAL (dpkind), PARAMETER :: p1 = .914041914819518E-09_dpkind
        REAL (dpkind), PARAMETER :: p2 = .238082361044469E-01_dpkind
        REAL (dpkind), PARAMETER :: q1 = -.499999999085958E+00_dpkind
        REAL (dpkind), PARAMETER :: q2 = .107141568980644E+00_dpkind
        REAL (dpkind), PARAMETER :: q3 = -.119041179760821E-01_dpkind
        REAL (dpkind), PARAMETER :: q4 = .595130811860248E-03_dpkind
! ..
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (IN) :: x
! ..
! .. Local Scalars ..
        REAL (dpkind) :: w
! ..
! .. Intrinsic Functions ..
        INTRINSIC ABS, EXP
! ..
! .. Function Return Value ..
        REAL (dpkind) :: expm1
! ..
        IF (ABS(x)<=0.15E0_dpkind) THEN
          expm1 = x*(((p2*x+p1)*x+one)/((((q4*x+q3)*x+q2)*x+q1)*x+one))
          RETURN
        END IF

        w = EXP(x)
        IF (x<=0.0E0_dpkind) THEN
          expm1 = (w-half) - half
          RETURN
        END IF

        expm1 = w*(half+(half-one/w))
        RETURN

      END FUNCTION expm1

!*********************************************************************

      SUBROUTINE fit_all_mu(seen,unseen,mu_init_est,mu_final_est, &
          mu_est_var)
! .. Array Arguments ..
        REAL (dpkind), OPTIONAL :: mu_est_var(:)
        REAL (dpkind) :: mu_final_est(:), mu_init_est(:), seen(:,:), &
          unseen(:,:)
! ..
! .. Local Scalars ..
        INTEGER :: i_allele
        LOGICAL :: calc_var
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
        calc_var = PRESENT(mu_est_var)

        DO i_allele = 1, n_allele

          IF (calc_var) THEN

            CALL fit_one_mu(seen(:,i_allele),unseen(:,i_allele), &
              mu_init_est(i_allele),mu_final_est(i_allele), &
              mu_est_var(i_allele))

          ELSE

            CALL fit_one_mu(seen(:,i_allele),unseen(:,i_allele), &
              mu_init_est(i_allele),mu_final_est(i_allele))

          END IF

        END DO

        RETURN

      END SUBROUTINE fit_all_mu

!*********************************************************************

      SUBROUTINE fit_one_mu(seen,unseen,mu_init_est,mu_final_est, &
          mu_est_var)
! .. Use Statements ..
        USE structures_mod, ONLY : dna
! ..
! .. Scalar Arguments ..
        REAL (dpkind), OPTIONAL, INTENT (OUT) :: mu_est_var
        REAL (dpkind), INTENT (OUT) :: mu_final_est, mu_init_est
! ..
! .. Array Arguments ..
        REAL (dpkind), INTENT (IN) :: seen(:), unseen(:)
! ..
! .. Local Scalars ..
        LOGICAL :: calc_var
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
        calc_var = PRESENT(mu_est_var)

        mu_init_est = initial_estimate(dna,seen,unseen)

        mu_final_est = refine_estimate(dna,seen,unseen,mu_init_est)

        IF (calc_var) THEN

          CALL ll_derivs(dna=dna,mu=mu_final_est,seen=seen,unseen=unseen, &
            d2ll_dmu2=mu_est_var)

          IF (mu_est_var<zero) THEN
            mu_est_var = -one/mu_est_var
          ELSE
            mu_est_var = zero
          END IF

        END IF

        RETURN

      END SUBROUTINE fit_one_mu

!*********************************************************************

      FUNCTION initial_estimate(dna,seen,unseen)
! .. Use Statements ..
        USE print_array_mod
! ..
! .. Function Return Value ..
        REAL (dpkind) :: initial_estimate
! ..
! .. Parameters ..
        REAL (dpkind), PARAMETER :: one = 1.0_dpkind
        REAL (dpkind), PARAMETER :: small_punseen = 1.0E-4_dpkind
! ..
! .. Local Arrays ..
        REAL (dpkind) :: punseen(n_dna), tmp(n_dna)
! ..
! .. Array Arguments ..
        REAL (dpkind), INTENT (IN) :: dna(n_dna), seen(n_dna), &
          unseen(n_dna)
! ..
! .. Intrinsic Functions ..
        INTRINSIC LOG, MAX, REAL, SUM
! ..
        punseen = MAX(unseen/(seen+unseen),small_punseen)

        tmp = -LOG(punseen)
        tmp = tmp/dna

        initial_estimate = SUM(tmp)/REAL(n_dna,dpkind)

        RETURN

      END FUNCTION initial_estimate

!*********************************************************************

      SUBROUTINE ll_derivs(dna,mu,seen,unseen,ll,dll_dmu,d2ll_dmu2)
! .. Scalar Arguments ..
        REAL (dpkind), OPTIONAL, INTENT (OUT) :: d2ll_dmu2, dll_dmu, ll
        REAL (dpkind), INTENT (IN) :: mu
! ..
! .. Array Arguments ..
        REAL (dpkind), INTENT (IN) :: dna(:), seen(:), unseen(:)
! ..
! .. Local Scalars ..
        INTEGER :: n_dna
! ..
! .. Intrinsic Functions ..
        INTRINSIC EXP, LOG, MAX, PRESENT, SIZE, SUM
! ..
        n_dna = SIZE(dna)

        IF (n_dna>max_dna) THEN
          WRITE (*,90000) max_dna, n_dna
90000     FORMAT (' This program only handles ',I4, &
            ' dna levels (ge)'/' The current call requests ',I4, &
            ' levels'/' The program is aborting.')
          STOP 'Too many dna_levels in setup_ll_derivs'
        END IF

        mean(1:n_dna) = mu*dna(1:n_dna)

        pseen(1:n_dna) = -expm1(-mean(1:n_dna))

        pseen(1:n_dna) = MAX(pseen(1:n_dna),tiny)

        punseen(1:n_dna) = EXP(-mean(1:n_dna))

        punseen_d_pseen(1:n_dna) = punseen(1:n_dna)/pseen(1:n_dna)

        IF (PRESENT(ll)) ll = SUM(seen*LOG(pseen( &
          1:n_dna))-unseen*mean(1:n_dna))

        IF (PRESENT(dll_dmu)) dll_dmu = SUM(dna*(seen*punseen_d_pseen( &
          1:n_dna)-unseen))

        IF (PRESENT(d2ll_dmu2)) d2ll_dmu2 = -SUM(seen*dna**2* &
          punseen_d_pseen(1:n_dna)/pseen(1:n_dna))

        RETURN

      END SUBROUTINE ll_derivs

!*********************************************************************

      FUNCTION refine_estimate(dna,seen,unseen,mu_init_est)
! .. Use Statements ..
        USE zero_finder
! ..
! .. Function Return Value ..
        REAL (dpkind) :: refine_estimate
! ..
! .. Array Arguments ..
        REAL (dpkind), INTENT (IN) :: dna(:), seen(:), unseen(:)
! ..
! .. Local Scalars ..
        REAL (dpkind) :: deriv, est
        INTEGER :: status
! ..
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (IN) :: mu_init_est
! ..
! .. Local Arrays ..
        REAL (dpkind) :: local_seen(SIZE(dna)), local_unseen(SIZE(dna))
! ..
! .. Intrinsic Functions ..
        INTRINSIC SIZE, SUM
! ..
        local_seen = seen

        local_unseen = unseen

!!! Fudge seen and unseen if necessary to assure that something is seen

        IF (SUM(local_seen)<tiny_mu) THEN

          local_seen(n_dna) = local_seen(n_dna) + hundredth

          local_unseen(n_dna) = local_unseen(n_dna) - hundredth

        END IF

        CALL set_zero_finder(low_limit=zero,hi_limit=huge_mu)

        status = 0

        est = mu_init_est

        DO

          CALL rc_step_zf(status,est,deriv)

          IF (status/=1) EXIT

          CALL ll_derivs(dna=dna,mu=est,seen=local_seen, &
            unseen=local_unseen,dll_dmu=deriv)

        END DO

        refine_estimate = est

        RETURN

      END FUNCTION refine_estimate

!*********************************************************************

    END MODULE fit_mu_mod
