    MODULE print_array_mod
!----------------------------------------------------------------------
!      SUBROUTINE print_vector(vector,lprint,format,n_per_line,unit)
!  A generic (double or integer) routine for printint a vector of
!  numbers.
!                       ARGUMENTS
!  Prints a one-dimensional array of double precision values on
!  a specified Fortran unit.
!                            ARGUMENTS
!  vector --> The numeric vector to be printed
!              REAL (dpkind), INTENT(IN) :: vector(:) or INTEGER :: vector(:)
!  lprint --> The number of elements to be printed
!             The entire array if lprint is ommitted
!              INTEGER, INTENT(IN), OPTIONAL :: lprint
!  format --> The format with which to print each element
!             If ommitted, 'F15.8' for double precision vector.
!             If ommitted, 'I10' for integer
!             Appropriate format differs depending on type of vector
!              CHARACTER(LEN=*), INTENT(IN), OPTIONAL :: format
!  n_per_line --> The number of elements per line to print
!             If ommitted, 5
!              INTEGER, INTENT(IN), OPTIONAL :: n_per_line
!  unit --> The unit number to which the vector is to be printed.
!           If ommitted, 6 is used (which usually is standard output)
!            INTEGER, INTENT(IN), OPTIONAL :: unit
!               +++++++++++++++++++++++++++++++++++++++++++++++++
!   SUBROUTINE print_matrix(matrix,nrow,ncol,format,n_per_line, unit)
!  A generic (double or integer) routine for printint a matrix of
!  numbers.
!                    ARGUMENTS
!  matrix --> The matrix to be printed
!        REAL (dpkind), INTENT (IN) :: matrix(:,:) or
!        INTEGER, INTENT(IN) :: matrix(:,:)
!  nrow --> If present the number of rows to print
!           If absent, all rows are printed
!           INTEGER, INTENT(IN), OPTIONAL :: nrow
!  ncol --> If present the number of columns to print
!           If absent, all columns are printed
!           INTEGER, INTENT(IN), OPTIONAL :: ncol
!  format --> The format with which to print each element
!             If ommitted, 'F15.8' for double precision vector.
!             If ommitted, 'I10' for integer
!             Appropriate format differs depending on type of vector
!              CHARACTER(LEN=*), INTENT(IN), OPTIONAL :: format
!  n_per_line --> The number of elements per line to print
!             If ommitted, 5
!              INTEGER, INTENT(IN), OPTIONAL :: n_per_line
!  unit --> The unit number to which the vector is to be printed.
!           If ommitted, 6 is used (which usually is standard output)
!            INTEGER, INTENT(IN), OPTIONAL :: unit
!----------------------------------------------------------------------
! .. Implicit None Statement ..
      IMPLICIT NONE
! ..
! .. Parameters ..
      INTEGER, PARAMETER, PRIVATE :: dpkind = KIND(0.0D0)
      INTEGER, PARAMETER, PRIVATE :: stdout = 6
! ..
! .. Generic Interface Blocks ..
      INTERFACE print_vector
        MODULE PROCEDURE print_vector_dbl, print_vector_int
      END INTERFACE
      INTERFACE print_matrix
        MODULE PROCEDURE print_matrix_dbl, print_matrix_int
      END INTERFACE
! ..
! .. Intrinsic Functions ..
      INTRINSIC KIND
! ..
    CONTAINS

!*********************************************************************

      SUBROUTINE print_matrix_dbl(matrix,nrow,ncol,format,n_per_line, &
          unit)
!----------------------------------------------------------------------
!  Prints a double precision matrix to standard output
!                            ARGUMENTS
!  matrix --> The matrix to be printed
!  nrow --> The number of rows to be printed
!           If ommitted all rows are printed
!  ncol --> The number of columns to be printed
!           If ommitted all columns are printed
!  format --> The format with which to print each element
!             If ommitted, 'F15.8'
!  n_per_line --> The number of elements per line to print
!             If ommitted, 4
!----------------------------------------------------------------------
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        INTEGER, OPTIONAL, INTENT (IN) :: ncol, nrow, n_per_line, unit
        CHARACTER (*), OPTIONAL, INTENT (IN) :: format
! ..
! .. Array Arguments ..
        REAL (dpkind), INTENT (IN) :: matrix(:,:)
! ..
! .. Local Scalars ..
        INTEGER :: hi, irow, ix, lo, local_n_per_line, local_unit, upcol, &
          uprow
        LOGICAL :: first
        CHARACTER (10) :: local_format, n_as_char
! ..
! .. Intrinsic Functions ..
        INTRINSIC MIN, PRESENT, SIZE
! ..
! .. Local Arrays ..
        CHARACTER (100) :: myformat(2)
! ..
! .. Executable Statements ..

        IF (PRESENT(nrow)) THEN
          uprow = nrow
        ELSE
          uprow = SIZE(matrix,1)
        END IF

        IF (PRESENT(ncol)) THEN
          upcol = ncol
        ELSE
          upcol = SIZE(matrix,2)
        END IF

        myformat = ' '
        n_as_char = ' '

        local_n_per_line = 5
        IF (PRESENT(n_per_line)) local_n_per_line = n_per_line

        local_format = 'F15.8'
        IF (PRESENT(format)) local_format = format

        IF (PRESENT(unit)) THEN
          local_unit = unit
        ELSE
          local_unit = stdout
        END IF

        WRITE (n_as_char,'(I10)') local_n_per_line

        myformat(1) = '('' ['',I3,''] '',' // n_as_char // &
          '( ''('',I3,'') '',' // local_format // ',:,2x))'

!!! ( ' [',I3,'] ', n ( '(',I3,') ', F12.4, 2X ) )


        myformat(2) = '(7X,' // n_as_char // '( ''('',I3,'') '',' // &
          local_format // ',:,2x))'


!        WRITE (local_unit,'(A)') myformat(1)
!        WRITE (local_unit,'(A)') myformat(2)

!!! ( 7X, n ( '(',I3,') ', F12.4, 2X ) )

        DO irow = 1, uprow

          lo = -local_n_per_line + 1

          hi = 0

          first = .TRUE.

          DO

            lo = lo + local_n_per_line

            IF (lo>upcol) EXIT

            hi = MIN(hi+local_n_per_line,upcol)

            IF (first) THEN

              first = .FALSE.

              WRITE (local_unit,myformat(1)) irow, &
                (ix,matrix(irow,ix),ix=lo,hi)

            ELSE

              WRITE (local_unit,myformat(2)) (ix,matrix(irow,ix),ix=lo, &
                hi)

            END IF

          END DO

        END DO

      END SUBROUTINE print_matrix_dbl

!*********************************************************************

      SUBROUTINE print_matrix_int(matrix,nrow,ncol,format,n_per_line, &
          unit)
!----------------------------------------------------------------------
!  Prints a double precision matrix to standard output
!                            ARGUMENTS
!  matrix --> The matrix to be printed
!  nrow --> The number of rows to be printed
!           If ommitted all rows are printed
!  ncol --> The number of columns to be printed
!           If ommitted all columns are printed
!  format --> The format with which to print each element
!             If ommitted, 'I10'
!  n_per_line --> The number of elements per line to print
!             If ommitted, 4
!----------------------------------------------------------------------
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        INTEGER, OPTIONAL, INTENT (IN) :: ncol, nrow, n_per_line, unit
        CHARACTER (*), OPTIONAL, INTENT (IN) :: format
! ..
! .. Array Arguments ..
        INTEGER, INTENT (IN) :: matrix(:,:)
! ..
! .. Local Scalars ..
        INTEGER :: hi, irow, ix, lo, local_n_per_line, local_unit, upcol, &
          uprow
        LOGICAL :: first
        CHARACTER (10) :: local_format, n_as_char
! ..
! .. Intrinsic Functions ..
        INTRINSIC MIN, PRESENT, SIZE
! ..
! .. Local Arrays ..
        CHARACTER (100) :: myformat(2)
! ..
! .. Executable Statements ..

        IF (PRESENT(nrow)) THEN
          uprow = nrow
        ELSE
          uprow = SIZE(matrix,1)
        END IF

        IF (PRESENT(ncol)) THEN
          upcol = ncol
        ELSE
          upcol = SIZE(matrix,2)
        END IF

        myformat = ' '
        n_as_char = ' '

        local_n_per_line = 5
        IF (PRESENT(n_per_line)) local_n_per_line = n_per_line

        local_format = 'I10'
        IF (PRESENT(format)) local_format = format


        IF (PRESENT(unit)) THEN
          local_unit = unit
        ELSE
          local_unit = stdout
        END IF

        WRITE (n_as_char,'(I10)') local_n_per_line

        myformat(1) = '('' ['',I3,''] '',' // n_as_char // &
          '( ''('',I3,'') '',' // local_format // ',:,2x))'

!!! ( ' [',I3,'] ', n ( '(',I3,') ', F12.4, 2X ) )


        myformat(2) = '(7X,' // n_as_char // '( ''('',I3,'') '',' // &
          local_format // ',:,2x))'

!!! ( 7X, n ( '(',I3,') ', F12.4, 2X ) )

!!!        WRITE (local_unit,'(A)') myformat(1)
!!!        WRITE (local_unit,'(A)') myformat(2)

        DO irow = 1, uprow

          lo = -local_n_per_line + 1

          hi = 0

          first = .TRUE.

          DO

            lo = lo + local_n_per_line

            IF (lo>upcol) EXIT

            hi = MIN(hi+local_n_per_line,upcol)

            IF (first) THEN

              first = .FALSE.

              WRITE (local_unit,myformat(1)) irow, &
                (ix,matrix(irow,ix),ix=lo,hi)

            ELSE

              WRITE (local_unit,myformat(2)) (ix,matrix(irow,ix),ix=lo, &
                hi)

            END IF

          END DO

        END DO

      END SUBROUTINE print_matrix_int

!*********************************************************************

      SUBROUTINE print_vector_dbl(vector,lprint,format,n_per_line,unit)
!----------------------------------------------------------------------
!  Prints a one-dimensional array of double precision values on
!  standard output (*)
!                            ARGUMENTS
!  vector --> The array to be printed
!  lprint --> The number of elements to be printed
!             The entire array if ommitted
!  format --> The format with which to print each element
!             If ommitted, 'F15.8'
!  n_per_line --> The number of elements per line to print
!             If ommitted, 5
!----------------------------------------------------------------------
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        INTEGER, OPTIONAL, INTENT (IN) :: lprint, n_per_line, unit
        CHARACTER (*), OPTIONAL, INTENT (IN) :: format
! ..
! .. Array Arguments ..
        REAL (dpkind), INTENT (IN) :: vector(:)
! ..
! .. Local Scalars ..
        INTEGER :: hi, ix, lo, local_n_per_line, local_unit, lvector
        CHARACTER (10) :: local_format, n_as_char
        CHARACTER (100) :: myformat
! ..
! .. Intrinsic Functions ..
        INTRINSIC MIN, PRESENT, SIZE
! ..
! .. Executable Statements ..

!!! Set up values for use
        IF ( .NOT. PRESENT(lprint)) THEN
          lvector = SIZE(vector)
        ELSE
          lvector = lprint
        END IF

        myformat = ' '
        n_as_char = ' '

        local_n_per_line = 5
        IF (PRESENT(n_per_line)) THEN
          local_n_per_line = n_per_line
        END IF

        local_format = 'F15.8'
        IF (PRESENT(format)) local_format = format

        IF (PRESENT(unit)) THEN
          local_unit = unit
        ELSE
          local_unit = stdout
        END IF

        WRITE (n_as_char,'(I10)') local_n_per_line

        myformat = '(1X,' // n_as_char // '(''('',I3,'') '',' // &
          local_format // ',:,2X))'

!!! (1X, n('(',I3,') ',F5.3,2X))

!        WRITE (local_unit,'(A)') myformat

        lo = -local_n_per_line + 1

        hi = 0

        DO

          lo = lo + local_n_per_line

          IF (lo>lvector) EXIT

          hi = MIN(hi+local_n_per_line,lvector)

          WRITE (local_unit,myformat) (ix,vector(ix),ix=lo,hi)

        END DO

      END SUBROUTINE print_vector_dbl

!*********************************************************************

      SUBROUTINE print_vector_int(vector,lprint,format,n_per_line,unit)
!----------------------------------------------------------------------
!  Prints a one-dimensional array of integer values on standard output
!  (*).
!                            ARGUMENTS
!  vector --> The array to be printed
!  lprint --> The number of elements to be printed
!             The entire array if ommitted
!  format --> The format with which to print each element
!             If ommitted, 'I10'
!  n_per_line --> The number of elements per line to print
!             If ommitted, 4
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        INTEGER, OPTIONAL, INTENT (IN) :: lprint, n_per_line, unit
        CHARACTER (*), OPTIONAL :: format
! ..
! .. Array Arguments ..
        INTEGER, INTENT (IN) :: vector(:)
! ..
! .. Local Scalars ..
        INTEGER :: hi, ix, lo, local_n_per_line, local_unit, lvector
        CHARACTER (10) :: local_format, n_as_char
        CHARACTER (100) :: myformat
! ..
! .. Intrinsic Functions ..
        INTRINSIC MIN, PRESENT, SIZE
! ..
! .. Executable Statements ..

!!! Set up values for use
        IF ( .NOT. PRESENT(lprint)) THEN
          lvector = SIZE(vector)
        ELSE
          lvector = lprint
        END IF

        myformat = ' '
        n_as_char = ' '

        local_n_per_line = 5
        IF (PRESENT(n_per_line)) local_n_per_line = n_per_line

        local_format = 'I10'
        IF (PRESENT(format)) local_format = format

        IF (PRESENT(unit)) THEN
          local_unit = unit
        ELSE
          local_unit = stdout
        END IF

        WRITE (n_as_char,'(I10)') local_n_per_line

        myformat = '(1X,' // n_as_char // '(''('',I3,'') '',' // &
          local_format // ',:,2X))'

!        WRITE (local_unit,'(A)') myformat

!!! (1X, n('(',I3,') ',I5,2X))

        lo = -local_n_per_line + 1

        hi = 0

        DO

          lo = lo + local_n_per_line

          IF (lo>lvector) EXIT

          hi = MIN(hi+local_n_per_line,lvector)

          WRITE (local_unit,myformat) (ix,vector(ix),ix=lo,hi)

        END DO

      END SUBROUTINE print_vector_int

!*********************************************************************

    END MODULE print_array_mod
