    MODULE results_out_mod
! .. Use Statements ..
      USE structures_mod
! ..
! .. Default Accessibility ..
      PRIVATE
! ..
! .. Public Statements ..
      PUBLIC :: print_answers, print_data, print_generate_params, &
        print_mu
! ..
! .. Parameters ..
      INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
! ..
!!! A repository for the routines that print the results
    CONTAINS

!*********************************************************************

      SUBROUTINE asterisks(rep_unit)
! .. Scalar Arguments ..
        INTEGER, OPTIONAL, INTENT (IN) :: rep_unit
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
        WRITE (*,90000)
        IF (PRESENT(rep_unit)) WRITE (rep_unit,90000)
90000   FORMAT (/5X,69('*')/)

      END SUBROUTINE asterisks

!*********************************************************************

      SUBROUTINE print_answers(rep_unit)
! .. Use Statements ..
        USE sppcr_aux_mod
        USE structures_mod
        USE print_it
        USE cdf_normal_mod
! ..
! .. Parameters ..
        INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
        CHARACTER (*), PARAMETER :: f104 = '(F10.4)', f84 = '(f8.4)', &
          f86 = '(F8.6)'
        REAL (dpkind), PARAMETER :: half = 0.5_dpkind
        REAL (dpkind), PARAMETER :: normal_975 = 1.959964_dpkind
! ..
! .. Local Scalars ..
        REAL (dpkind) :: hi_bnd, lo_bnd
        INTEGER :: i_allele
        LOGICAL :: print_rep
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: rep_unit
! ..
        print_rep = rep_unit > 0


!!! CALIBRATION PRINT OUT

        CALL asterisks(rep_unit)

        WRITE (*,'(T30,''CALIBRATION'')')

        IF (print_rep) WRITE (rep_unit,'(T30,''CALIBRATION'')')

        CALL asterisks(rep_unit)

        lo_bnd = calibrate(data,est) - normal_975*calibrate(boot,sd)
        hi_bnd = calibrate(data,est) + normal_975*calibrate(boot,sd)

        WRITE (*,90000) calibrate(data,est), lo_bnd, hi_bnd, &
          one/calibrate(data,est), one/hi_bnd, one/lo_bnd

        IF (print_rep) WRITE (rep_unit,90000) calibrate(data,est), &
          lo_bnd, hi_bnd, one/calibrate(data,est), one/hi_bnd, one/lo_bnd

90000   FORMAT (5X,'One experimenter''s units is ',F8.4, &
          ' genome equivalents'/T15,'95% CI ( ',F8.4,', ',F8.4,' )'/5X, &
          'One genome equivalent is ',F8.4,' experimenter''s units'/T15, &
          '95% CI ( ',F8.4,', ',F8.4,' )'/)
90100   FORMAT (//5X,'Estimate of calibration constant:',T60,F8.4/5X, &
          'Mean bootstrap estimate of calibration constant:',T60, &
          F8.4//5X,'Asymptotic estimate of SE of calibration constant:', &
          T60,F8.4/5X,'Bootstrap estimate of SE of calibration constant:' &
          ,T60,F8.4//)


        WRITE (6,90100) calibrate(data,est), calibrate(boot,est), &
          calibrate(data,sd), calibrate(boot,sd)
        IF (print_rep) WRITE (rep_unit,90100) calibrate(data,est), &
          calibrate(boot,est), calibrate(data,sd), calibrate(boot,sd)

!!! TOTAL MUTANT FREQUENCY

        CALL asterisks(rep_unit)

        WRITE (*,90200)
        IF (print_rep) WRITE (rep_unit,90200)

90200   FORMAT (T19,'TOTAL FREQUENCY OF MUTANT ALLELES')

        CALL asterisks(rep_unit)

        lo_bnd = inv_arcsin(mutant(data,trans,est)- &
          normal_975*mutant(boot,trans,sd))

        hi_bnd = inv_arcsin(mutant(data,trans,est)+ &
          normal_975*mutant(boot,trans,sd))

        WRITE (*,'(T23,''Untransformed Frequency''/)')
        IF (print_rep) WRITE (rep_unit, &
          '(T23,''Untransformed Frequency''/)')

        WRITE (*,90300) mutant(data,untrans,est), &
          mutant(data,untrans,sd), mutant(boot,untrans,est), &
          mutant(boot,untrans,sd)
        IF (print_rep) WRITE (rep_unit,90300) mutant(data,untrans,est), &
          mutant(data,untrans,sd), mutant(boot,untrans,est), &
          mutant(boot,untrans,sd)

        WRITE (*,'(T20,''Arc_sin Transformed Frequency''/)')
        IF (print_rep) WRITE (rep_unit, &
          '(T20,''Arc_sin Transformed Frequency''/)')

        WRITE (*,90300) mutant(data,trans,est), mutant(data,trans,sd), &
          mutant(boot,trans,est), mutant(boot,trans,sd)
        IF (print_rep) WRITE (rep_unit,90300) mutant(data,trans,est), &
          mutant(data,trans,sd), mutant(boot,trans,est), &
          mutant(boot,trans,sd)

90300   FORMAT (T6,'Estimate:',T32,F8.6,T42,'Asymptotic SE:',T57,F8.6/T6, &
          'Mean Bootstrap Estimate:',T32,F8.6,T42,'Bootstrap SE: ',T57, &
          F8.6/)

        WRITE (*,90400) lo_bnd, hi_bnd
        IF (print_rep) WRITE (rep_unit,90400) lo_bnd, hi_bnd

90400   FORMAT (5X,'95% Confidence Limits: ( ',F8.4,', ',F8.4,' )'/T22, &
          'From Transformed Bootstrap Values')

!!! Test for difference in frequency of progenitor alleles.  Should
!!! use the mu's instead of the freq values since the freq values are
!!! correlated.  At the moment just get rid of this.

!         num_subs = 2

!         sub_pos(1:4) = (/ 98, 105, 133, 140 /)

! ! Assign sub_strings(1:2)

! !!! Test for evidence of difference in proportions of the two progenitor
! !!! alleles if there are two

!         IF (progenitor(1)/=progenitor(2)) THEN

!           z = ABS(freq(progenitor(1),boot,trans,est)-freq(progenitor( &
!             2),boot,trans,est))/SQRT(freq(progenitor(1),boot,trans,sd)**2 &
!             +freq(progenitor(2),boot,trans,sd)**2)

!           p_value = cum_normal(-z)

! ! sub_string_len = 8
!           WRITE (sub_strings(1),f86) z

! ! sub_string_len = 10
!           WRITE (sub_strings(2),f86) p_value

!           message_format = '(5X,''   +++++++++++++++ TEST DIFFERENCE &
!             &IN PROGENITOR FREQUENCIES +++++++++++++++''//5X,''Z-score: &
!             &%%%%%%%%                  P-Value: %%%%%%%%''/)'

!           IF (print_rep) THEN
!             CALL print_message_format(force=.TRUE.,unit=rep_unit)
!           ELSE
!             CALL print_message_format(force=.TRUE.)
!           END IF

!         END IF

        CALL asterisks(rep_unit)

        WRITE (*,'(T14,''FREQUENCY ESTIMATES OF INDIVIDUAL ALLELES'')')

        CALL asterisks(rep_unit)

        num_subs = 0

        message_format = '(/5X,''Allele    Estimate  Mean  &
          &    Asymptotic  Bootstrap Lo 95%   Hi 95%''/5X,''Size &
          &     of        Bootstrap  Standard    Standard  &
          &Conf     Conf''/5X,''          Frequency Estimate &
          & Error       Error''/)'

        IF (print_rep) THEN
          CALL print_message_format(force=.TRUE.,unit=rep_unit)
        ELSE
          CALL print_message_format(force=.TRUE.)
        END IF

        num_subs = 8

        sub_pos(1:16) = (/ 6, 10, 13, 13, 16, 23, 26, 33, 36, 43, 48, 55, &
          58, 65, 67, 74 /)

! Assign sub_strings(1:8)

DETAIL: DO i_allele = 1, n_allele

! sub_string_len = 5  --allele size

          WRITE (sub_strings(1),'(I5)') allele_size(i_allele)

          IF ((i_allele==progenitor(1)) .OR. (i_allele==progenitor(2))) &
              THEN
            sub_strings(2) = 'P'
          ELSE
            sub_strings(2) = 'M'
          END IF

! sub_string_len = 1 -- P or M for progenitor or mutant

! Frequency raw estimate

! sub_string_len = 8 -- estimate from data
          WRITE (sub_strings(3),f86) freq(i_allele,data,untrans,est)

! sub_string_len = 8 -- mean bootstrap estimate
          WRITE (sub_strings(4),f86) freq(i_allele,boot,untrans,est)

! sub_string_len = 8 -- asy SD
          WRITE (sub_strings(5),f86) freq(i_allele,data,untrans,sd)

! sub_string_len = 8 -- bootstrap SE
          WRITE (sub_strings(6),f86) freq(i_allele,boot,untrans,sd)

          lo_bnd = inv_arcsin(freq(i_allele,data,trans,est)- &
            normal_975*freq(i_allele,boot,trans,sd))

          hi_bnd = inv_arcsin(freq(i_allele,data,trans,est)+ &
            normal_975*freq(i_allele,boot,trans,sd))

! sub_string_len = 8 -- lo conf
          WRITE (sub_strings(7),f86) lo_bnd

! sub_string_len = 8
          WRITE (sub_strings(8),f86) hi_bnd

          message_format = '(5X,''%%%%%  %  %%%%%%%%  %%%%%%%% &
            & %%%%%%%%    %%%%%%%%  %%%%%%%% %%%%%%%%'')'


          IF (print_rep) THEN
            CALL print_message_format(force=.TRUE.,unit=rep_unit)
          ELSE
            CALL print_message_format(force=.TRUE.)
          END IF

        END DO DETAIL

        CALL asterisks(rep_unit)
        CALL asterisks(rep_unit)

      END SUBROUTINE print_answers

!*********************************************************************

      SUBROUTINE print_data(rep_unit)
! .. Local Scalars ..
        REAL (dpkind) :: cal_con
        INTEGER :: i_dna
        LOGICAL :: print_rep
! ..
! .. Intrinsic Functions ..
        INTRINSIC INT
! ..
! .. Parameters ..
        REAL (dpkind), PARAMETER :: half = 0.5_dpkind
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: rep_unit
! ..
        cal_con = calibrate(data,est)

        print_rep = rep_unit > 0

        CALL asterisks(rep_unit)

        WRITE (*,'(T31,''DATA'')')
        IF (print_rep) WRITE (rep_unit,'(T31,''DATA'')')

        WRITE (*,90000) n_allele, n_dna
        IF (print_rep) WRITE (rep_unit,90000) n_allele, n_dna

90000   FORMAT ('Number of alleles: ',I3/'Number of runs (dna amounts): ' &
          ,I3/)

        WRITE (*,90100) allele_size(progenitor(1)), &
          allele_size(progenitor(2))
        IF (print_rep) WRITE (rep_unit,90100) allele_size(progenitor(1)), &
          allele_size(progenitor(2))

90100   FORMAT ('Sizes of progenitor alleles: ','( ',I3,' , ',I3, &
          ' )'//T25,'Number of Times Each Allele Seen'/)

        WRITE (*,90200) allele_size
        IF (print_rep) WRITE (rep_unit,90200) allele_size

90200   FORMAT ('  DNA     DNA     N'/'  Exp     G.E.   Well'/'  Unit'// &
          T35,'ALLELE SIZE'//(T25,13(I3,1X):/))

        DO i_dna = 1, n_dna

          WRITE (*,90300) dna(i_dna), cal_con*dna(i_dna), &
            INT(n_well(i_dna)), INT(seen(i_dna,:,data))

          IF (print_rep) WRITE (rep_unit,90300) dna(i_dna), &
            cal_con*dna(i_dna), INT(n_well(i_dna)), &
            INT(seen(i_dna,:,data))

90300     FORMAT (F6.2,2X,F6.2,T18,I3,(T25,13(I3,1X):/))
! 90300     FORMAT (F5.1,T8,I3,(T13,15(I3,1X)))

        END DO

      END SUBROUTINE print_data

!*********************************************************************

      SUBROUTINE print_generate_params(rep_unit)
! .. Use Statements ..
        USE structures_mod
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: rep_unit
! ..
! .. Local Scalars ..
        INTEGER :: i
        LOGICAL :: print_rep
! ..
! .. Intrinsic Functions ..
        INTRINSIC INT
! ..
        print_rep = rep_unit > 0

        CALL asterisks(rep_unit)

        WRITE (*,'(T17,''PARAMETERS ENTERED FOR GENERATING DATA'')')
        IF (print_rep) WRITE (rep_unit, &
          '(T17,''PARAMETERS ENTERED FOR GENERATING DATA'')')

        CALL asterisks(rep_unit)

        WRITE (*,90000) n_dna, n_allele
        IF (print_rep) WRITE (rep_unit,90000) n_dna, n_allele

90000   FORMAT (' The number of runs (dna amounts) simulated is: ', &
          I3//' The number of alleles simulated is: ',I3//)

        WRITE (*,90100) INT(n_well(1))
        IF (print_rep) WRITE (rep_unit,90100) INT(n_well(1))
90100   FORMAT (' The number of wells per run is the same for all runs.'/ &
          ' The common  number of wells is: ',I5//)

        WRITE (*,90200)
        IF (print_rep) WRITE (rep_unit,90200)

90200   FORMAT (22X,'Allele Frequencies Used to Generate the Data'//)

        WRITE (*,90300) (i,true_freq(i),i=1,n_allele)
        IF (print_rep) WRITE (rep_unit,90300) (i,true_freq(i),i=1, &
          n_allele)
90300   FORMAT ((5('(',I3,')',1X,F8.6,2X)))

        WRITE (*,90400) true_calibrate
        IF (print_rep) WRITE (rep_unit,90400) true_calibrate

90400   FORMAT (// &
          ' The calibration factor is the number of genome equivalents'/ &
          ' in one experimenter''s unit.'// &
          ' The calibration factor used to generate the data is: ',F8.3)

        WRITE (*,90500) progenitor
        IF (print_rep) WRITE (rep_unit,90500) progenitor
90500   FORMAT (/' The indices of the progenitor alleles are: ',2I4//)

        RETURN

      END SUBROUTINE print_generate_params

!*********************************************************************

      SUBROUTINE print_mu(rep_unit)
! .. Use Statements ..
        USE structures_mod
        USE print_it
! ..
! .. Parameters ..
        INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
        CHARACTER (*), PARAMETER :: f104 = '(F10.4)', f84 = '(f8.4)', &
          f86 = '(F8.6)'
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: rep_unit
! ..
! .. Local Scalars ..
        INTEGER :: i_allele
        LOGICAL :: print_rep
! ..
        print_rep = rep_unit > 0

        CALL asterisks(rep_unit)

        WRITE (*, &
          '(T12,''Mu[i]: Calibration Times Frequency of Allele i'')')
        IF (print_rep) WRITE (rep_unit, &
          '(T12,''Mu[i]: Calibration Times Frequency of Allele i'')')

        num_subs = 0

        message_format = '(/5X,''     Allele    Initial   Final &
          &    Mean      Asymptotic  Bootstrap''/5X,''     &
          &Size      Estimate  Estimate  Bootstrap Standard &
          &   Standard''/5X,''                             &
          &      Estimate  Error       Error''/)'

        IF (print_rep) THEN
          CALL print_message_format(force=.TRUE.,unit=rep_unit)
        ELSE
          CALL print_message_format(force=.TRUE.)
        END IF

        num_subs = 6

        sub_pos(1:12) = (/ 14, 16, 21, 28, 31, 38, 41, 48, 51, 58, 63, &
          70 /)

        DO i_allele = 1, n_allele

! sub_string_len = 3
          WRITE (sub_strings(1),'(I3)') allele_size(i_allele) ! Allele label

! sub_string_len = 8
          WRITE (sub_strings(2),f84) mu(i_allele,data,init_est)

! sub_string_len = 8
          WRITE (sub_strings(3),f84) mu(i_allele,data,est)

! sub_string_len = 8
          WRITE (sub_strings(4),f84) mu(i_allele,boot,est)

! sub_string_len = 8
          WRITE (sub_strings(5),f84) mu(i_allele,data,sd)

! sub_string_len = 8
          WRITE (sub_strings(6),f84) mu(i_allele,boot,sd)

          message_format = '(5X,''        %%%    %%%%%%%%  &
            &%%%%%%%%  %%%%%%%%  %%%%%%%%    %%%%%%%%'')'

          IF (print_rep) THEN
            CALL print_message_format(force=.TRUE.,unit=rep_unit)
          ELSE
            CALL print_message_format(force=.TRUE.)
          END IF

        END DO

      END SUBROUTINE print_mu

!*********************************************************************

    END MODULE results_out_mod
