    MODULE set_seeds_mod
! .. Use Statements ..
      USE ecuyer_cote_mod
      USE get_numbers_mod
! ..
! .. Implicit None Statement ..
      IMPLICIT NONE
! ..
! .. Default Accessibility ..
      PRIVATE
! ..
! .. Local Scalars ..
      INTEGER :: seed1, seed2
      CHARACTER (500) :: message_format
      CHARACTER (80) :: phrase
! ..
! .. Public Statements ..
      PUBLIC :: set_seeds
! ..
    CONTAINS

!*********************************************************************

      SUBROUTINE phrase_set_seeds
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. External Subroutines ..
        EXTERNAL phrase_to_seed
! ..
! .. Executable Statements ..

        message_format = '(/5X,''Enter a  phrase that will &
          & be used to initialize  the random''/5X,''number &
          & generator.    Different  phrases  provide  different''/5X,''in&
          &itialization values.''/)'

        WRITE (*,message_format)

        READ (*,'(A)') phrase

        CALL phrase_to_seed(phrase,seed1,seed2)

        CALL set_all_seeds(seed1,seed2)

        RETURN

      END SUBROUTINE phrase_set_seeds

!*********************************************************************

      SUBROUTINE set_seeds(which)
!!! Set random number seeds from time (if which == 1) else from
!!! user entered phrase (if which /=1)
!!! If which is ommitted, ask user what to do
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        INTEGER, OPTIONAL, INTENT (IN) :: which
! ..
! .. Local Scalars ..
        INTEGER :: local_which
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
! .. Executable Statements ..

        IF (PRESENT(which)) THEN

          local_which = which

        ELSE

          message_format = '(/5X,''Enter (1) to set random &
            &number seeds from the time of day''/5X,''     &
            &     (Computer runs so set cannot be replicated &
            &exactly)''//5X,''      (2) to set seeds from a &
            &phrase (character string) entered''/5X,''     &
            &     by you''/5X,''          (Computer runs so &
            &set can be replicated exactly)''/)'

          CALL get_numbers(message_format,lo=1,hi=2,x=local_which)

        END IF

        IF (local_which==1) THEN
          CALL time_set_seeds
        ELSE
          CALL phrase_set_seeds
        END IF

        RETURN

      END SUBROUTINE set_seeds

!*********************************************************************

      SUBROUTINE time_set_seeds
!!! Sets random number generator randomly from the clock
!!! Assumes that there is a clock.
!!! Warns if there is apparently no clock
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Local Scalars ..
        CHARACTER (10) :: time
! ..
! .. Intrinsic Functions ..
        INTRINSIC DATE_AND_TIME
! ..
! .. External Subroutines ..
        EXTERNAL phrase_to_seed
! ..
! .. Executable Statements ..

        CALL DATE_AND_TIME(time=time)

        IF (time==' ') THEN

!!! Apparently no clock

          message_format = '(/5X,''There appears to  be no &
            &accessible clock on  this computer, hence''/5X,''the &
            &random  number generator seed cannot be  initialized &
            &from the''/5X,''time.  You must thus enter a phrase &
            &(character string) to set the''/5X,''seed.''/)'

          WRITE (*,message_format)

          CALL phrase_set_seeds

          RETURN

        ELSE

          CALL phrase_to_seed(time,seed1,seed2)

          CALL set_all_seeds(seed1,seed2)

        END IF

        RETURN

      END SUBROUTINE time_set_seeds

!*********************************************************************

    END MODULE set_seeds_mod
