    MODULE structures_mod
!!!----------------------------------------------------------------------
!!!
!!! ALLELE_SIZE Indices
!!!
!!!      (1) i_allele BOUNDS: 1, n_allele
!!! DNA Indices
!!!
!!!      (1) i_dna BOUNDS: 1, n_dna
!!!
!!! CALIBRATE Indices
!!!
!!!      (1) data, boot
!!!
!!!      (2) est, var, sd, esum, ess
!!!
!!! FREQ Indices
!!!
!!!      (1) i_allele BOUNDS(1:n_allele)
!!!
!!!      (2) data, boot
!!!
!!!      (3) untrans, trans
!!!
!!!      (4) est, var, sd, esum, ess
!!!
!!! MU Indices
!!!
!!!      (1) i_allele BOUNDS(1:n_allele)
!!!
!!!      (2) data, boot
!!!
!!!      (3) init_est, est, var, sd, esum, ess
!!!
!!! MUTANT Indices
!!!
!!!      (1) data, boot
!!!
!!!      (2) untrans, trans
!!!
!!!      (3) est, var, sd, esum, ess
!!!
!!! SEEN, UNSEEN Indices
!!!
!!!      (1) i_dna BOUNDS: 1, n_dna
!!!
!!!      (2) i_allele BOUNDS: 1, n_allele
!!!
!!!      (3) data, boot, shrink
!!!
!!!----------------------------------------------------------------------
! .. Default Accessibility ..
      PUBLIC
! ..
! .. Private Statements ..
      PRIVATE :: dpkind, zero
! ..
! .. Parameters ..
      INTEGER, PARAMETER :: boot = 2, data = 1, ess = 5, est = 1, &
        esum = 4, init_est = 0, sd = 3, shrink = 3, trans = 2, &
        untrans = 1, var = 2
      INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
      REAL (dpkind), PARAMETER :: zero = 0.0_dpkind
! ..
! .. Local Scalars ..
      REAL (dpkind) :: true_calibrate
      INTEGER :: n_allele, n_dna, prev_n_allele, prev_n_dna
      LOGICAL :: first = .TRUE., n_allele_same = .FALSE., &
        n_dna_same = .FALSE.
! ..
! .. Local Arrays ..
      REAL (dpkind) :: calibrate(data:boot,est:ess), &
        mutant(data:boot,untrans:trans,est:ess)
      REAL (dpkind), ALLOCATABLE :: dna(:), freq(:,:,:,:), mu(:,:,:), &
        n_well(:), seen(:,:,:), true_freq(:), unseen(:,:,:)
      INTEGER, ALLOCATABLE :: allele_size(:)
      INTEGER :: progenitor(2)
! ..
! .. Intrinsic Functions ..
      INTRINSIC KIND
! ..
    CONTAINS

!*********************************************************************

      SUBROUTINE initialize

FIRST_CONDITION: IF (first) THEN

          first = .FALSE.

          ALLOCATE (freq(1:n_allele,data:boot,untrans:trans,est:ess))

          ALLOCATE (mu(1:n_allele,data:boot,init_est:ess))

          ALLOCATE (seen(1:n_dna,1:n_allele,data:shrink), &
            unseen(1:n_dna,1:n_allele,data:shrink))

          ALLOCATE (dna(1:n_dna),n_well(1:n_dna))

          ALLOCATE (allele_size(1:n_allele))

          ALLOCATE (true_freq(1:n_allele))

        ELSE FIRST_CONDITION

          IF (n_allele/=prev_n_allele) THEN

            ALLOCATE (allele_size(1:n_allele))

            DEALLOCATE (freq,mu,seen,unseen,allele_size)


            ALLOCATE (freq(1:n_allele,data:boot,untrans:trans,est:ess))

            ALLOCATE (mu(1:n_allele,data:boot,init_est:ess))

            ALLOCATE (seen(1:n_dna,1:n_allele,data:shrink), &
              unseen(1:n_dna,1:n_allele,data:shrink))

            ALLOCATE (true_freq(1:n_allele))

          END IF

          IF (n_dna/=prev_n_dna) THEN

            ALLOCATE (dna(1:n_dna))

            DEALLOCATE (seen,unseen,dna)

            ALLOCATE (seen(1:n_dna,1:n_allele,data:shrink), &
              unseen(1:n_dna,1:n_allele,data:shrink))

          END IF

        END IF FIRST_CONDITION

        first = .FALSE.

        prev_n_dna = n_dna

        prev_n_allele = n_allele

        freq = zero

        mu = zero

        calibrate = zero

        mutant = zero

      END SUBROUTINE initialize

!*********************************************************************

    END MODULE structures_mod
