#include "zerofinder.h"
#include "suh.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

double interval_mu=.5;
int interval_neps = 100;
double small_step=.0001;
double absolute_tol;
double relative_tol;
double absolute_stepsize;
double relative_stepsize;
double step_multiplier=2.0;
double converge=.000001;
double infinity;
int step_from;
double step_fsmall;
double commonbig;
double commonsmall;
int common_current_status;
char common_called_from_step;
char common_crash_left;
char common_crash_hi;
double step_fbig;
char step_qincr;
char step_qup;
char zf_crash_left;
char zf_crash_hi;
char step_qok;
char step_qlim;
char step_qcond;
char step_qbdd;
int zf_current_status;
double step_xsave;
double step_step;
double step_yy;
double step_xub;
double step_xlb;
double common_zf_bound_low;
double common_zf_bound_high;
double zf_bound_low;
double zf_bound_high;

double interval_init_f = 100000;

double interval_a;
double interval_a0;
double interval_b;
double interval_b0;
double interval_c;
double interval_d;
double interval_e;
double interval_u;
double interval_fa;
double interval_fb;
double interval_fc;
double interval_fd;
double interval_fe;
double interval_fu;
double interval_tol0;
double interval_tol;
double interval_eps;
double interval_lenab;
double interval_lambda = .7;
int interval_from;
int interval_itnum;

void setZeroFinder(double low,double high)
{
	infinity=pow(10,20);
	commonsmall=low;
	commonbig=high;
	absolute_tol=converge;
	relative_tol=converge;
	absolute_stepsize=small_step;
	relative_stepsize=1.0;
}

void setZeroFinderDetailed(double low, double high, double absstep, double relstep, double stepmul, double abtol, double reltol)
{
	infinity=pow(10,20);
	commonsmall=low;
	commonbig=high;
	absolute_tol=abtol;
	relative_tol=reltol;
	absolute_stepsize=absstep;
	relative_stepsize=relstep;
	step_multiplier=stepmul;
}

// est = x, deriv = fx
void rc_step_zf(int *status,double *est,double deriv)
{
	if(*status==1)
	{
		switch(step_from)
		{
			case 1:
				goto lbl10;
			case 2:
				goto lbl20;
			case 3:
				goto lbl30;
			case 4:
				goto lbl50;
			case 5:
				goto lbl80;
			case 6:
				goto lbl100;
		}
	}
	if(*status!=0)
	{
		printf("illegal input of status: %d\n",*status);
		exit(1);
	}
	common_current_status=*status;
	if(!qxmon(commonsmall,*est,commonbig))
	{
		printf("*est(%lf) is out of bound\n",*est);
		exit(1);
	}
	step_xsave=*est;
	// see that small and big bound the zero and set qincr
	*est=commonsmall;
	step_from=1;
	*status=1;
	common_current_status=*status;
	return;
lbl10:
	step_fsmall=deriv;
	*est=commonbig;
	step_from=2;
	*status=1;
	common_current_status=*status;
	return;
lbl20:
	step_fbig=deriv;
	step_qincr=step_fbig > step_fsmall;
	if(step_qincr)
	{
		if(!(step_fsmall<=0))
		{
			*status=-1;
			zf_current_status=-1;
			zf_crash_left=true;
			zf_crash_hi=true;
			common_current_status=*status;
			common_crash_left=zf_crash_left;
			common_crash_hi=zf_crash_hi;
			return;
		}
		if(!(step_fbig>=0))
		{
			*status=-1;
			zf_current_status=-1;
			zf_crash_left=false;
			zf_crash_hi=false;
			return;
		}
	}
	else
	{
		if(!(step_fsmall>=0))
		{
			*status=-1;
			zf_current_status=-1;
			zf_crash_left=true;
			zf_crash_hi=false;
			return;
		}
		if(!(step_fbig<=0))
		{
			*status=-1;
			zf_current_status=-1;
			zf_crash_left=false;
			zf_crash_hi=true;
			return;
		}
	}
	*est=step_xsave;
	step_step=absolute_stepsize;
	if(relative_stepsize*fabs(*est)>step_step)
		step_step=relative_stepsize*fabs(*est);
	step_from=3;
	*status=1;
	return;
lbl30:
	step_yy=deriv;
	if(step_yy==0)
	{
		*status=0;
		zf_current_status=0;
		step_qok=true;
		return;
	}
	step_qup=(step_qincr && step_yy<0) || (!step_qincr && step_yy>0);
	if(!step_qup)
		goto lbl60;
	step_xlb=step_xsave;
	step_xub=step_xlb+step_step;
	if(step_step>commonbig)
		step_step=commonbig;
lbl40:
	*est=step_xub;
	step_from=4;
	*status=1;
	return;
lbl50:
	step_yy=deriv;
	step_qbdd=(step_qincr && step_yy>=0) || (!step_qincr && step_yy<=0);
	step_qlim=step_xub>=commonbig;
	step_qcond=step_qbdd || step_qlim;
	if(!step_qcond)
	{
		step_step=step_multiplier*step_step;
		step_xlb=step_xub;
		step_xub=step_xlb+step_step;
		if(step_step>commonbig)
			step_step=commonbig;
		goto lbl40;
	}
	if(step_qlim && !step_qbdd)
	{
		*status=-1;
		zf_current_status=-1;
		zf_crash_left=false;
		zf_crash_hi=!step_qincr;
		*est=commonbig;
		return;
	}
	goto lbl90;
lbl60:
	step_xub=step_xsave;
	step_xlb=step_xub-step_step;
	if(step_step<commonsmall)
		step_step=commonsmall;
lbl70:
	*est=step_xlb;
	step_from=5;
	*status=1;
	return;
lbl80:
	step_yy=deriv;
	step_qbdd=(step_qincr && step_yy<=0) || (!step_qincr && step_yy>=0);
	step_qlim=step_xlb<=commonsmall;
	step_qcond=step_qbdd || step_qlim;
	if(!step_qcond)
	{
		step_step=step_multiplier*step_step;
		step_xub=step_xlb;
		step_xlb=step_xub-step_step;
		if(step_step<commonsmall)
			step_step=commonsmall;
		goto lbl70;
	}
	if(step_qlim && !step_qbdd)
	{
		*status=-1;
		zf_current_status=-1;
		zf_crash_left=true;
		zf_crash_hi=step_qincr;
		*est=commonsmall;
		return;	
	}
lbl90:
	*status=0;
	zf_current_status=0;
	common_called_from_step=true;
lbl100:
	rc_interval_zf(status,est,deriv);
	step_from=6;
	if(*status!=1)
		status=0;
	return;
}

int qxmon(double zx,double zy,double zz)
{
	return zx<= zy && zy<=zz;
}

// the main code
void rc_interval_zf(int *status,double *est,double deriv)
{
	int iprof;
	
	if(*status==0)
	{
		// assumes set zero finder HAS been called
		if(common_called_from_step)
		{
			interval_a = step_xlb;
			interval_b=step_xub;
		}
		else
		{
			interval_a = commonsmall;
			interval_b = commonbig;
		}
		interval_tol=0;
		interval_itnum=0;
		interval_e = interval_init_f;
		interval_fe= interval_init_f;
		//interval_eps = EPSILON(one)
		//interval_eps= nextafter(0,1);
		interval_eps=1.1920929*pow(10,-7);
		// ignore that and use the nextafter() function
		if(interval_neps>300)
			interval_tol0=0;
		else
			interval_tol0=1.0/pow(10,interval_neps);//interval_tol0=1.0/pow(10,interval_neps);
		*est = interval_a;
		*status = 1;
		interval_from=1;
		return;
	}
	if(*status==1)
	{
		switch(interval_from)
		{
			case 1:
				interval_fa=deriv;
				*est = interval_b;
				*status = 1;
				interval_from=2;
				return;
			case 2:
				interval_fb = deriv;
				if(interval_fa<0 && interval_fb<0)
				{
					*status=-1;
					zf_current_status=-1;
					zf_crash_left=interval_fa<interval_fb;
					zf_crash_hi=false;
					return;
				}
				else if(interval_fa>0 && interval_fb>0)
				{
					*status=-1;
					zf_current_status=-1;
					zf_crash_left=interval_fa>interval_fb;
					zf_crash_hi = true;
					return;
				}
				interval_a0=interval_a;
				interval_b0 = interval_b;
				interval_lenab = interval_b-interval_a;
				interval_itnum++;
				ftol();
				if(interval_lenab<=interval_tol)
				{
					goto lbl100;
				}
				if(interval_itnum!=1)
				{
					if(interval_itnum==2 || ifabde()==0)
						newton_quadratic(2);
					else
					{
					    pzero();
						if(iisign(interval_c-interval_a)*iisign(interval_c-interval_b)>=0)
							newton_quadratic(2);
					}
					interval_e=interval_d;
					interval_fe=interval_fd;
					adjust_c();
					*est = interval_c;
					interval_from = 4;
					*status=1;
					return;
				}
				interval_c=interval_a-(interval_fa/(interval_fb-interval_fa))*interval_lenab;
				adjust_c();
				*est=interval_c;
				interval_from=3;
				*status=1;
				return;
			case 3:
				interval_fc = deriv;		// lbl 40
				bracket();
				if(hasconverged())
				{
					goto lbl100;
				}
				else
				{
					interval_a0=interval_a;		// lbl 30
					interval_b0 = interval_b;
					interval_lenab = interval_b-interval_a;
					interval_itnum++;
					ftol();
					if(interval_lenab<=interval_tol)
					{
						goto lbl100;
					}
					if(interval_itnum!=1)
					{
						if(interval_itnum==2 || ifabde()==0)	// lbl 50
							newton_quadratic(2);
						else
						{
							pzero();
							if(iisign(interval_c-interval_a)*iisign(interval_c-interval_b)>=0)
								newton_quadratic(2);
						}
						interval_e=interval_d;
						interval_fe=interval_fd;
						adjust_c();
						*est = interval_c;
						interval_from = 4;
						*status=1;
						return;
					}
					interval_c=interval_a-(interval_fa/(interval_fb-interval_fa))*interval_lenab;
					adjust_c();
					*est=interval_c;
					interval_from=3;
					*status=1;
					return;
				}   
			
			case 4:
				interval_fc=deriv;	// lbl 60
				bracket();
				if(hasconverged())
				{
					goto lbl100;
				}
				if(ifabde()==0)
					newton_quadratic(3);
				else
				{
					pzero();
					
					iprof = iisign(interval_c - interval_a)*iisign(interval_c-interval_b);
					if(iprof>=0)
						newton_quadratic(3);
				}
				adjust_c();
				*est = interval_c;
				interval_from=5;
				*status=1;
				return;
			
			case 5:
				interval_fc=deriv; // lbl 70
				bracket();
				if(hasconverged())
				{
					goto lbl100;
				}
				interval_e=interval_d;
				interval_fe=interval_fd;
				// double size secant steps, versus small granny steps
				if(fabs(interval_fa)<fabs(interval_fb))
				{
					interval_u=interval_a;
					interval_fu=interval_fa;
				}
				else
				{
					interval_u=interval_b;
					interval_fu=interval_fb;
				}
				interval_c=interval_u-2*(interval_fu/(interval_fb-interval_fa))*interval_lenab;
				
				if(fabs(interval_c-interval_u)>.5*(interval_a+interval_b))
					interval_c=.5*(interval_a+interval_b);
				adjust_c();
				*est=interval_c;
				interval_from=6;
				*status=1;
				return;
			
			case 6:
				interval_fc=deriv; // lbl 80
				bracket();
				if(hasconverged())
				{
					*status=0;				// lbl 100
					zf_current_status=0;
					zf_bound_low=interval_a;
					zf_bound_high=interval_b;
					common_zf_bound_low=zf_bound_low;
					common_zf_bound_high=zf_bound_high;
					*est = .5 * (zf_bound_low + zf_bound_high);
					return;
				}
				if(interval_lenab<interval_mu*(interval_b0-interval_a0))
				{
					interval_a0=interval_a;		// lbl 30
					interval_b0 = interval_b;
					interval_lenab = interval_b-interval_a;
					interval_itnum++;
					ftol();
					if(interval_lenab<=interval_tol)
					{
						goto lbl100;
					}
					if(interval_itnum!=1)
					{
						if(interval_itnum==2 || ifabde()==0)	// lbl 50
							newton_quadratic(2);
						else
						{
							pzero();
							if(iisign(interval_c-interval_a)*iisign(interval_c-interval_b)>=0)
								newton_quadratic(2);
						}
						interval_e=interval_d;
						interval_fe=interval_fd;
						adjust_c();
						*est = interval_c;
						interval_from = 4;
						*status=1;
						return;
					}
					interval_c=interval_a-(interval_fa/(interval_fb-interval_fa))*interval_lenab;
					adjust_c();
					*est=interval_c;
					interval_from=3;
					*status=1;
					return;
				}
				interval_e=interval_d;
				interval_fe=interval_fd;
				interval_c=.5*(interval_a+interval_b);
				adjust_c();
				
				*est=interval_c;
				interval_from=7;
				*status=1;
				return;
				
			case 7:
				interval_fc=deriv;
				bracket();
				if(hasconverged())
				{
					goto lbl100;
				}
				else
				{
					interval_a0=interval_a;		// lbl 30
					interval_b0 = interval_b;
					interval_lenab = interval_b-interval_a;
					interval_itnum++;
					ftol();
					if(interval_lenab<=interval_tol)
					{
						goto lbl100;
					}
					if(interval_itnum!=1)
					{
						if(interval_itnum==2 || ifabde()==0)	// lbl 50
							newton_quadratic(2);
						else
						{
							pzero();
							if(iisign(interval_c-interval_a)*iisign(interval_c-interval_b)>=0)
								newton_quadratic(2);
						}
						interval_e=interval_d;
						interval_fe=interval_fd;
						adjust_c();
						*est = interval_c;
						interval_from = 4;
						*status=1;
						return;
					}
					interval_c=interval_a-(interval_fa/(interval_fb-interval_fa))*interval_lenab;
					adjust_c();
					*est=interval_c;
					interval_from=3;
					*status=1;
					return;
				}
				
			default:
				printf("interval_from = %d",interval_from);
				exit(1);
		}
	}
	if(*status!=0)
	{
		printf("status = %d\nillegal input values of status in rc_interval_zf",*status);
		exit(1);
	}
	interval_fa=deriv;
	*est=interval_b;
	*status=1;
	interval_from=2;
	return;
lbl100:
	*status = 0;
	zf_current_status= 0;
	zf_bound_low = interval_a;
	zf_bound_high = interval_b;
	common_zf_bound_low = zf_bound_low;
	common_zf_bound_high = zf_bound_high;
	*est = .5*(zf_bound_low+zf_bound_high);

	return;
}

void ftol()	// checked out
{
	if(fabs(interval_fb)<=fabs(interval_fa))
		interval_tol= 2*(interval_tol0 + 2*fabs(interval_b)*interval_eps);
	else
		interval_tol= 2*(interval_tol0 + 2*fabs(interval_a)*interval_eps);
}

int ifabde()	// checked out
{
	return iisign(interval_fa-interval_fb)*
	iisign(interval_fa-interval_fd)*
	iisign(interval_fa-interval_fe)*
	iisign(interval_fb-interval_fd)*
	iisign(interval_fb-interval_fe)*
	iisign(interval_fd-interval_fe);
}

int iisign(double x)	// checked out
{
	if(x>0)
		return 1;
	if(x<0)
		return -1;
	return 0;
}
void newton_quadratic(int k)
{
	int ierror=0,x;
	double a0=interval_fa;
	double a1=(interval_fb-interval_fa)/interval_lenab;
	double a2=((interval_fd-interval_fb)/(interval_d-interval_b)-a1)/(interval_d-interval_a);
	if(interval_d-interval_a==0)
	{
		printf("divide by 0\n");
		exit(1);
	}
	double pc,pdc;
lblten:
	if(a2==0 || ierror==1)
	{
		interval_c=interval_a - a0/a1;
		return;
	}
	if(iisign(a2)*iisign(interval_fa)>0)
	{
		interval_c=interval_a;
	}
	else
	{
		interval_c=interval_b;
	}
	for(x=0;x<k;x++)
	{
		if(ierror==0)
		{
			pc=a0+(a1+a2*(interval_c-interval_b))*(interval_c-interval_a);
			pdc=a1 + a2*((2*interval_c)-(interval_a+interval_b));
			if(pdc==0)
				ierror=1;
			else
			{
				interval_c-=pc/pdc;
			}
		}
	}
	if(ierror==1)
		goto lblten;
	
	return;
}

void pzero()	// checked out
{
	double q32,d32,q33;
	double q11 = (interval_d-interval_e)*(interval_fd/(interval_fe-interval_fd));
	double q21 = (interval_b-interval_d)*(interval_fb/(interval_fd-interval_fb));
	double cons = -interval_lenab/(interval_fb-interval_fa);
	double q31 = cons * interval_fa;
	double d21 = (interval_b - interval_d) * (interval_fd/(interval_fd-interval_fb));
	double d31 = cons * interval_fb;
	double q22 = (d21-q11)*(interval_fb/(interval_fe-interval_fb));
	cons = (d31-q21)/(interval_fd-interval_fa);
	q32 = cons * interval_fa;
	d32 = cons * interval_fd;
	q33 = (d32-q22)*(interval_fa/(interval_fe - interval_fa));
	
	interval_c = interval_a + q31 + q32 + q33;  // that's it?

}

// adjusts c if b-a is very small or if c is close to a or b
void adjust_c()	// function checked out
{
	interval_tol*=interval_lambda;
	if(interval_lenab<= 2*interval_tol)
	{
		interval_c=.5*(interval_a+interval_b);
	}
	else if(interval_c<=(interval_a + interval_tol))
	{
		interval_c=interval_a+interval_tol;
	}
	else
	{
		if(interval_c>=interval_b-interval_tol)
			interval_c = interval_b-interval_tol;
	}
}

void bracket()	// function checked out
{
	if(interval_fc==0)
	{
		interval_a=interval_c;
		interval_fa=0;
		interval_d=0;
		interval_fd=0;
		return;
	}
	if(iisign(interval_fa)*iisign(interval_fc)<0)
	{
		interval_d=interval_b;
		interval_fd=interval_fb;
		interval_b=interval_c;
		interval_fb=interval_fc;
	}
	else
	{
		interval_d=interval_a;
		interval_fd=interval_fa;
		interval_a=interval_c;
		interval_fa=interval_fc;
	}
	interval_lenab=interval_b-interval_a;
	ftol();
}

int hasconverged()	// checked out
{
	return interval_fa==0 || interval_lenab<=interval_tol;

}